/* Copyright (c) 2014 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
*
* @defgroup gpiote_example_main main.c
* @{
* @ingroup nrf_gpiote_example
* @brief GPIOTE Example Application main file.
*
* This file contains the source code for a sample application using GPIOTE.
*/

#include <stdbool.h>
#include <stdint.h>
#include "nrf.h"
#include "nrf_gpiote.h"
#include "nrf_gpio.h"
#include "boards.h"
#include "nrf_drv_ppi.h"
#include "nrf_drv_timer.h"
#include "nrf_drv_gpiote.h"
#include "app_error.h"
#include "app_timer.h"
#include "nrf_drv_clock.h"
#include "nrf_delay.h"

#define PERIOD_0 200
#define PERIOD_1 300
#define PERIOD_2 400

#define APP_TIMER_PRESCALER             15    // Value of the RTC1 PRESCALER register.
#define APP_TIMER_OP_QUEUE_SIZE         3     // Size of timer operation queues.

APP_TIMER_DEF(m_led_0_timer_id);
APP_TIMER_DEF(m_led_1_timer_id);
APP_TIMER_DEF(m_led_2_timer_id);

static void lfclk_request(void)
{
    uint32_t err_code = nrf_drv_clock_init();
    APP_ERROR_CHECK(err_code);
    nrf_drv_clock_lfclk_request(NULL);
}

static void timer_0_handler(void * p_context)
{
    nrf_drv_gpiote_out_toggle(BSP_LED_0);
}

static void timer_1_handler(void * p_context)
{
    nrf_drv_gpiote_out_toggle(BSP_LED_1);
}

static void timer_2_handler(void * p_context)
{
    nrf_drv_gpiote_out_toggle(BSP_LED_2);
}


static void create_timers()
{   
    uint32_t err_code;

    // Create timers
    err_code = app_timer_create(&m_led_0_timer_id,
                                APP_TIMER_MODE_REPEATED,
                                timer_0_handler);
    APP_ERROR_CHECK(err_code);
    
    err_code = app_timer_create(&m_led_1_timer_id,
                                APP_TIMER_MODE_REPEATED,
                                timer_1_handler);
    APP_ERROR_CHECK(err_code);
    
    err_code = app_timer_create(&m_led_2_timer_id,
                                APP_TIMER_MODE_REPEATED,
                                timer_2_handler);
    APP_ERROR_CHECK(err_code);
}

static void led_blinking_setup()
{

    ret_code_t err_code;

    nrf_drv_gpiote_out_config_t out_config = GPIOTE_CONFIG_OUT_SIMPLE(false);

    err_code = nrf_drv_gpiote_out_init(BSP_LED_0, &out_config);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_gpiote_out_init(BSP_LED_1, &out_config);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_gpiote_out_init(BSP_LED_2, &out_config);
    APP_ERROR_CHECK(err_code);

}


/**
 * @brief Function for application main entry.
 */
int main(void)
{
    ret_code_t err_code;
    
    lfclk_request();
    
    err_code = nrf_drv_gpiote_init();
    APP_ERROR_CHECK(err_code);
    
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, false);

    led_blinking_setup();

    // Create application timers.
    create_timers();
    
    // Start application timers
    err_code = app_timer_start(m_led_0_timer_id, APP_TIMER_TICKS(PERIOD_0, APP_TIMER_PRESCALER), NULL);
    APP_ERROR_CHECK(err_code);
    err_code = app_timer_start(m_led_1_timer_id, APP_TIMER_TICKS(PERIOD_1, APP_TIMER_PRESCALER), NULL); 
    APP_ERROR_CHECK(err_code);
    err_code = app_timer_start(m_led_2_timer_id, APP_TIMER_TICKS(PERIOD_2, APP_TIMER_PRESCALER), NULL);
    APP_ERROR_CHECK(err_code);

    while (true)
    {
//        nrf_drv_gpiote_out_toggle(BSP_LED_3);
//        nrf_delay_ms(100);
    }
}


/** @} */
